from django.contrib.auth.models import AnonymousUser, AbstractUser, Group, Permission
from django.db.models.manager import EmptyManager


class JWTUser(AbstractUser):
    mapfre_id = 999
    pk = None
    username = 'JWT user'
    is_staff = False
    is_active = True
    is_superuser = False
    _groups = EmptyManager(Group)
    _user_permissions = EmptyManager(Permission)

    def get_mapfre_id(self):
        return self.mapfre_id

    def __str__(self):
        return 'JWTUser mapfre_id={} username={}'.format(self.mapfre_id, self.username)

    def __eq__(self, other):
        return isinstance(other, self.__class__)

    def __hash__(self):
        return 1  # instances always return the same hash value

    def save(self, **kwargs):
        raise NotImplementedError("Django doesn't provide a DB representation for AnonymousUser.")

    def delete(self, **kwargs):
        raise NotImplementedError("Django doesn't provide a DB representation for AnonymousUser.")

    def set_password(self, raw_password):
        raise NotImplementedError("Django doesn't provide a DB representation for AnonymousUser.")

    def check_password(self, raw_password):
        raise NotImplementedError("Django doesn't provide a DB representation for AnonymousUser.")

    @property
    def groups(self):
        return self._groups

    @property
    def user_permissions(self):
        return self._user_permissions

    def get_group_permissions(self, obj=None):
        return set()

    def get_all_permissions(self, obj=None):
        return None

    def has_perm(self, perm, obj=None):
        return None

    def has_perms(self, perm_list, obj=None):
        for perm in perm_list:
            if not self.has_perm(perm, obj):
                return False
        return True

    def has_module_perms(self, module_name):
        return None

    @property
    def is_anonymous(self):
        return False

    @property
    def is_authenticated(self):
        return True

    def get_username(self):
        return self.username
